package com.qmth.teachcloud.exchange.common.service;

import com.qmth.teachcloud.exchange.common.bean.dto.syssetting.SimpleObject;
import com.qmth.teachcloud.exchange.common.entity.BasicSchool;

import java.util.Map;

/**
 * @Description: 缓存操作service
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/25
 */
public interface CommonCacheService {

    /**
     * 添加学校缓存
     *
     * @param schoolId
     * @return
     */
    public BasicSchool schoolCache(Long schoolId);

    /**
     * 添加学校缓存
     *
     * @param code
     * @return
     */
    public BasicSchool schoolCache(String code);

    /**
     * 删除学校缓存
     */
    public void removeSchoolIdCache();

    /**
     * 删除学校缓存
     */
    public void removeSchoolCodeCache();

    /**
     * 删除系统参数数据
     *
     * @param schoolId
     * @return
     */
    String removeSysSetting(Long schoolId);

    /**
     * 获取学校缓存
     */
    Map<String, SimpleObject> getSysSetting(Long schoolId);
}
